import { select,
        scaleLinear,
        axisLeft,
        axisBottom 
       } from 'd3';

export const barChart = (selection, props) => {
	const {
    yValue,
    xValue,
    xAxisLabel,
   	margin,
    width,
    height,
    data
  } = props
  
  let div = d3.select('#tooltip').append('text')
        .attr('class', 'tooltip')
        .style('display', 'none');
  function mouseover(){
    div.style('display', 'inline');
  }
  function mousemove(){
    let d = d3.select(this).data()[0]
    div
    	.html(d.key + ': ' + d.value)
      .style('left', (d3.event.pageX - 34) + 'px')
      .style('top', (d3.event.pageY - 12) + 'px');
  }
  function mouseout(){
    div.style('display', 'none');
  }

	const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = d3.scaleBand()
    .range([0, innerWidth])
    .domain(data.map((d) => d.key))
    .padding(0.3)
  
  const yScale = d3.scaleLinear()
    .range([innerHeight, 0])
    .domain([0, d3.max(data, yValue)])
		.nice();
  
  
  const g = selection.selectAll('.contain').data([null]);
  const gEnter = g.enter().append('g')
  .attr('class', 'contain');
  
  gEnter
    .merge(g)
  		.attr('transform',
            `translate(${margin.left},${margin.top})`
           );
  
  const xAxis = axisBottom(xScale)
  	//.tickSize(-innerHeight)
  	//.tickPadding(20);
  
 	const yAxis = axisLeft(yScale)
  	//.tickFormat(axisTickFormat)
    .tickSize(-innerWidth)
  	.tickPadding(10);
  
  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
  	.append('g')
  		.attr('class', 'y-axis')
  yAxisG
  	.merge(yAxisGEnter)
  		.call(yAxis);
  
  yAxisG
    .merge(yAxisGEnter)
    .selectAll('.domain').remove();
  
  const yAxisLabelText = yAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', 90)//-93)
      .attr('fill', 'black')
      .attr('transform', `rotate(90)`)
      .style('text-anchor', 'middle')
    .merge(yAxisG.select('.axis-label'))
      .attr('x', innerHeight/2)
      .text('absolute Häufigkeit');
  
  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
  	.append('g')
  		.attr('class', 'x-axis')
  xAxisG
  	.merge(xAxisGEnter)
  		.attr('transform', `translate(0, ${innerHeight})`)
  		.call(xAxis)
  	.selectAll('.tick text')
    	.style('text-anchor', 'end')
  		.attr("dx", "-.8em")
      .attr("dy", ".15em")
  		.attr("transform", function(d) {
                return "rotate(-30)" 
                });
  
  xAxisG
    .merge(xAxisGEnter)
    .selectAll('.domain').remove();
  
  const xAxisLabelText = xAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', 75)
      .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
      .attr('x', innerWidth / 2)
      .text(xAxisLabel);
  
  const titleG = g.select('.title');
  const titleGEnter = gEnter
  	.append('g')
  	.attr('class', 'title')
  
  const titleText = titleGEnter
  	.append('text')
  		.attr('class', 'title')
  		.attr('x', 0)
  		.attr('fill', 'black')
  	.merge(titleG.select('.title'))
  		.attr('y', -margin.top / 2)
  		.text('Balkendiagramm der Spalte: ' + xAxisLabel)
  
  const rectangles = g.merge(gEnter).selectAll('rect').data(data);
  rectangles.enter().append('rect')
  	.merge(rectangles)
  	.on('mouseover', mouseover)
    .on('mousemove', mousemove)
    .on('mouseout', mouseout)
      .attr('class', 'bar')
      .attr('x', d => xScale(d.key))
      .attr('y', d => yScale(yValue(d)))
      .attr('height', d => innerHeight - yScale(yValue(d)))
      .attr('width', xScale.bandwidth())
  	.on('mouseenter', function(actual, i){
    		//Alle Rechtecke werden durchsichtig
  			d3.selectAll('rect')
    			.attr('opacity', 0.7)
    		//Value der Bar wird unsichtbar wenn über Bar gehovert
    		d3.selectAll('.barValue')
    			.attr('opacity', 0)
    		//gehoverte Bar wird ausgewählt und undurchsichtig gemacht
    		d3.select(this)
    			.attr('opacity', 1)
    			//Animation
          .transition()
          .duration(300)
  				//gehoverte Bar wird dicker
          //.attr('x', d => xScale(d.key) - 5)
          //.attr('width', xScale.bandwidth() + 10)
    		
    		const y = yScale(actual.value)
        
        //gibt erlaubte Anzahl an Nachkommastellen an für Differenzwerte
        const fixedValue = 0
        
        const divergenceText = g.merge(gEnter).selectAll('.divergence').data(data);
    		divergenceText.enter().append('text')
            .attr('class', 'divergence')
            .attr('fill', 'white')
            .attr('text-anchor', 'middle')
    			.merge(divergenceText)
    				.attr('x', d => xScale(d.key) + xScale.bandwidth() / 2)
          	.attr('y', d => yScale(d.value) - 5)
    				.text((d, idx) => {
            const divergence = (d.value - actual.value).toFixed(fixedValue)
            
            let text = ''
            if (divergence > 0) text += '+'
            text += `${divergence}`

            return idx !== i ? text : '';
        		})
  	})
  	.on('mouseleave', function () {
    	//alle Bars wieder undurchsichtig
  			d3.selectAll('rect')
  				.attr('opacity', 1)
  		//Value der Bar wird sichtbar gemacht
        d3.selectAll('.barValue')
          .attr('opacity', 1)
			
  		//gehoverte Rechteck wieder auf normal Größe schrumpfen
        d3.select(this)
          .transition()
          .duration(300)
          //.attr('opacity', 1)
          //.attr('x', d => xScale(d.key))
          //.attr('width', xScale.bandwidth())

  			//Differenzwerte löschen
        d3.selectAll('.divergence').remove()
  	})
  rectangles.exit().remove()
  
  const barValueText = g.merge(gEnter).selectAll('.barValue').data(data);
  barValueText.enter().append('text')
      .attr('text-anchor', 'middle')
  	.merge(barValueText)
  		.attr('class', 'barValue')
  		.attr('x', d => xScale(d.key) + xScale.bandwidth() / 2)
      .attr('y', d => yScale(d.value) - 5)
      .text( d => `${d.value}`)
  barValueText.exit().rem